# -*- coding: utf-8 -*-
import datetime

from httpretty import httpretty

from rwslib import RWSConnection, rws_requests, AuthorizationException

__author__ = 'glow'

import unittest
try:
    import mock
except ImportError:
    from unittest import mock

import requests
from rwslib.rwsobjects import RWSPostResponse, RWSSubjects, RWSSubjectListItem, \
    RWSStudyMetadataVersions, RWSStudies, RWSResponse, RWSException
from rwslib.rws_requests import RWSRequest, StudySubjectsRequest, check_dataset_type, SubjectDatasetRequest, \
    VersionDatasetRequest, StudyDatasetRequest, PostDataRequest, PostMetadataRequest, \
    GlobalLibraryVersionRequest, GlobalLibraryVersionsRequest, GlobalLibraryDraftsRequest, \
    GlobalLibrariesRequest, StudyVersionRequest, StudyVersionsRequest, StudyDraftsRequest, \
    MetadataStudiesRequest, ClinicalStudiesRequest, CacheFlushRequest, DiagnosticsRequest, \
    BuildVersionRequest, CodeNameRequest, TwoHundredRequest


class TestRWSRequest(unittest.TestCase):
    class FooRWSRequest(RWSRequest):
            method = "POST"
            requires_authorization = True

            def __init__(self, project_name):
                self.project_name = project_name

    def test___eq___on_RWSRequest(self):
        self.assertEqual(RWSRequest(), RWSRequest())

    def test___eq___same_subclass(self):
        request1 = TestRWSRequest.FooRWSRequest('project')
        request2 = TestRWSRequest.FooRWSRequest('project')
        self.assertEqual(request1, request2)

    def test___eq___different_subclass(self):
        class BarRWSRequest(TestRWSRequest.FooRWSRequest):
            pass
        request1 = TestRWSRequest.FooRWSRequest('project')
        request2 = BarRWSRequest('project')
        self.assertNotEqual(request1, request2)

    def test___ne__(self):
        request1 = TestRWSRequest.FooRWSRequest('project1')
        request2 = TestRWSRequest.FooRWSRequest('project2')
        self.assertNotEqual(request1, request2)


class TestStudySubjectsRequest(unittest.TestCase):
    def setUp(self):
        self.project_name = "A Project"
        self.environment = "Dev"

    def test_subj_key_type_is_valid(self):
        """
        Confirm that only valid SubjectKeyTypes are accepted
        """
        with self.assertRaises(ValueError) as err:
            request = StudySubjectsRequest(self.project_name, self.environment,
                                           subject_key_type="AnApple")
        self.assertEqual(str(err.exception), "SubjectKeyType AnApple is not a valid value")
        request = StudySubjectsRequest(self.project_name, self.environment,
                                       subject_key_type="SubjectName")
        self.assertEqual(self.project_name, request.project_name)

    def test_request_uuid_type(self):
        """
        Confirm we request the UUID Subject Key Type
        """
        request = StudySubjectsRequest(self.project_name, self.environment,
                                       subject_key_type="SubjectUUID")
        self.assertTrue("subjectKeyType=SubjectUUID" in request.url_path())

    def test_request_without_type(self):
        """
        Confirm we request the UUID Subject Key Type
        """
        request = StudySubjectsRequest(self.project_name, self.environment)
        self.assertFalse("subjectKeyType" in request.url_path())

    def test_request_with_status(self):
        """
        Confirm we request the status when asked to
        """
        request = StudySubjectsRequest(self.project_name, self.environment, status=True)
        self.assertTrue("status=all" in request.url_path())

    def test_request_with_include(self):
        """
        Confirm we request the status when asked to
        """
        request = StudySubjectsRequest(self.project_name, self.environment, include="inactive")
        self.assertTrue("include=inactive" in request.url_path())
        request = StudySubjectsRequest(self.project_name, self.environment,
                                       include="inactiveAndDeleted")
        self.assertTrue("include=inactiveAndDeleted" in request.url_path())
        request = StudySubjectsRequest(self.project_name, self.environment, include="deleted")
        self.assertTrue("include=deleted" in request.url_path())
        with self.assertRaises(ValueError) as err:
            request = StudySubjectsRequest(self.project_name, self.environment,
                                           include="kitchen_sink")
        self.assertEqual(
            "If provided, included must be one of inactive,deleted,inactiveAndDeleted",
            str(err.exception))

    def test_maps_response(self):
        """We map the response to a RWSSubjects instance"""
        response_content = u"""    <ODM FileType="Snapshot"
         FileOID="770f1758-db33-4ab2-af72-38db863734aa"
         CreationDateTime="2013-04-08T14:08:06.875-00:00"
         ODMVersion="1.3">

         <ClinicalData StudyOID="Fixitol(Dev)" MetaDataVersionOID="1111">
            <SubjectData SubjectKey="000001">
               <SiteRef LocationOID="BP001"/>
            </SubjectData>
         </ClinicalData>

         <ClinicalData StudyOID="Fixitol(Dev)" MetaDataVersionOID="1111">
             <SubjectData SubjectKey="1111">
                <SiteRef LocationOID="335566"/>
             </SubjectData>
         </ClinicalData>
    </ODM>
"""
        request = StudySubjectsRequest("Fixitol",
                                       "Dev")
        req = mock.Mock(requests.Request, text=response_content)
        response = request.result(req)
        self.assertTrue(isinstance(response, RWSSubjects))
        for subject in response:
            self.assertTrue(isinstance(subject, RWSSubjectListItem))
            self.assertTrue(subject.subject_name in ['000001', "1111"])


class TestCheckDatasetType(unittest.TestCase):
    def test_with_valid_input(self):
        """We match when the input is what we expect it to be"""
        for dataset_type in ['regular', 'raw', 'REGULAR', 'RAW']:
            try:
                check_dataset_type(dataset_type)
            except ValueError:
                self.fail("Dataset {0} should be valid".format(dataset_type))

    def test_with_invalid_input(self):
        """We error with incorrect dataset_type"""
        for dataset_type in ['ruler', 'pencil', 'cheese']:
            with self.assertRaises(ValueError) as exc:
                check_dataset_type(dataset_type)
            self.assertEqual("Dataset type not 'regular' or 'raw' is %s" % dataset_type,
                             str(exc.exception))


class TestSubjectDatasetRequest(unittest.TestCase):
    def create_request_object(self, **kwargs):
        """
        Helper class
        :param args:
        :return: a SubjectDatasetRequest
        """
        default = dict(project_name="Mediflex",
                       environment_name="Prod",
                       subjectkey="1001",
                       dataset_type='regular',
                       start=None,
                       rawsuffix=None,
                       formoid=None,
                       versionitem=None,
                       codelistsuffix=None,
                       decodesuffix=None,
                       stdsuffix=None)
        default.update(kwargs)
        t = SubjectDatasetRequest(**default)
        return t

    def test_default_subjects_path(self):
        """We create a SubjectDatasetRequest with the defaults"""
        t = self.create_request_object()
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/subjects/1001/datasets/regular", t.url_path())

    def test_raw_subjects_path(self):
        """We create a SubjectDatasetRequest with raw"""
        t = self.create_request_object(dataset_type="raw")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/subjects/1001/datasets/raw", t.url_path())

    def test_raw_subjects_path_with_form(self):
        """We create a SubjectDatasetRequest with raw"""
        t = self.create_request_object(dataset_type="raw", formoid="DM")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/subjects/1001/datasets/raw/DM", t.url_path())

    def test_banana_subjects_path(self):
        """We create a SubjectDatasetRequest with nonsense"""
        with self.assertRaises(ValueError) as exc:
            t = self.create_request_object(dataset_type="pineapple")
        self.assertEqual("Dataset type not 'regular' or 'raw' is pineapple",
                         str(exc.exception))

    def test_create_start_using_datetime(self):
        """We can pass a datetime instance as the start argument"""
        jan = datetime.datetime(year=2012, month=12, day=1, hour=12, minute=12, second=23)
        t = self.create_request_object(dataset_type="raw", formoid="DM", start=jan)
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/subjects/1001/datasets/raw/DM?start=2012-12-01T12%3A12%3A23", t.url_path())

    def test_create_start_using_string_datetime(self):
        """We can pass a string (iso8601) instance as the start argument"""
        t = self.create_request_object(dataset_type="raw", formoid="DM", start="2012-12-01T12:12:23")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/subjects/1001/datasets/raw/DM?start=2012-12-01T12%3A12%3A23", t.url_path())

    def test_create_start_using_string_date(self):
        """We can pass a string (iso8601) instance as the start argument"""
        t = self.create_request_object(dataset_type="raw", formoid="DM", start="2012-12-01")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/subjects/1001/datasets/raw/DM?start=2012-12-01",
                         t.url_path())


class TestVersionDatasetRequest(unittest.TestCase):
    def create_request_object(self, **kwargs):
        """
        Helper class
        :param args:
        :return: a VersionDatasetRequest
        """
        default = dict(project_name="Mediflex",
                       environment_name="Prod",
                       version_oid='1001',
                       dataset_type='regular',
                       start=None,
                       rawsuffix=None,
                       formoid=None,
                       versionitem=None,
                       codelistsuffix=None,
                       decodesuffix=None,
                       stdsuffix=None)
        default.update(kwargs)
        t = VersionDatasetRequest(**default)
        return t

    def test_default_versions_path(self):
        """We create a VersionDatasetRequest with the defaults"""
        t = self.create_request_object()
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/versions/1001/datasets/regular", t.url_path())

    def test_raw_versions_path(self):
        """We create a VersionDatasetRequest with raw"""
        t = self.create_request_object(dataset_type="raw")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/versions/1001/datasets/raw", t.url_path())

    def test_raw_versions_path_with_form(self):
        """We create a SubjectDatasetRequest with raw"""
        t = self.create_request_object(dataset_type="raw", formoid="DM")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/versions/1001/datasets/raw/DM", t.url_path())

    def test_banana_versions_path(self):
        """We create a VersionDatasetRequest with nonsense"""
        with self.assertRaises(ValueError) as exc:
            t = self.create_request_object(dataset_type="pineapple")
        self.assertEqual("Dataset type not 'regular' or 'raw' is pineapple",
                        str(exc.exception))

    def test_create_start_using_datetime(self):
        """We can pass a datetime instance as the start argument"""
        jan = datetime.datetime(year=2012, month=12, day=1, hour=12, minute=12, second=23)
        t = self.create_request_object(dataset_type="raw", formoid="DM", start=jan)
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/versions/1001/datasets/raw/DM?start=2012-12-01T12%3A12%3A23", t.url_path())

    def test_create_start_using_string_datetime(self):
        """We can pass a string (iso8601) instance as the start argument"""
        t = self.create_request_object(dataset_type="raw", formoid="DM", start="2012-12-01T12:12:23")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/versions/1001/datasets/raw/DM?start=2012-12-01T12%3A12%3A23", t.url_path())

    def test_create_start_using_string_date(self):
        """We can pass a string (iso8601) instance as the start argument"""
        t = self.create_request_object(dataset_type="raw", formoid="DM", start="2012-12-01")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/versions/1001/datasets/raw/DM?start=2012-12-01", t.url_path())


class TestStudyDatasetRequest(unittest.TestCase):
    def create_request_object(self, **kwargs):
        """
        Helper class
        :param args:
        :return: a VersionDatasetRequest
        """
        default = dict(project_name="Mediflex",
                       environment_name="Prod",
                       dataset_type='regular',
                       start=None,
                       rawsuffix=None,
                       formoid=None,
                       versionitem=None,
                       codelistsuffix=None,
                       decodesuffix=None,
                       stdsuffix=None)
        default.update(kwargs)
        t = StudyDatasetRequest(**default)
        return t

    def test_default_versions_path(self):
        """We create a StudyDatasetRequest with the defaults"""
        t = self.create_request_object()
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/datasets/regular", t.url_path())

    def test_raw_versions_path(self):
        """We create a StudyDatasetRequest with raw"""
        t = self.create_request_object(dataset_type="raw")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/datasets/raw", t.url_path())

    def test_raw_versions_path_query_param(self):
        """We create a StudyDatasetRequest with raw and a query param"""
        t = self.create_request_object(dataset_type="raw", rawsuffix="RX")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/datasets/raw?rawsuffix=RX", t.url_path())

    def test_raw_versions_path_with_form(self):
        """We create a StudyDatasetRequest with raw"""
        t = self.create_request_object(dataset_type="raw", formoid="DM")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/datasets/raw/DM", t.url_path())

    def test_banana_versions_path(self):
        """We create a StudyDatasetRequest with nonsense"""
        with self.assertRaises(ValueError) as exc:
            t = self.create_request_object(dataset_type="pineapple")
        self.assertEqual("Dataset type not 'regular' or 'raw' is pineapple",
                         str(exc.exception))

    def test_create_start_using_datetime(self):
        """We can pass a datetime instance as the start argument"""
        jan = datetime.datetime(year=2012, month=12, day=1, hour=12, minute=12, second=23)
        t = self.create_request_object(dataset_type="raw", formoid="DM", start=jan)
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/datasets/raw/DM?start=2012-12-01T12%3A12%3A23", t.url_path())

    def test_create_start_using_string_datetime(self):
        """We can pass a string (iso8601) instance as the start argument"""
        t = self.create_request_object(dataset_type="raw", formoid="DM", start="2012-12-01T12:12:23")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/datasets/raw/DM?start=2012-12-01T12%3A12%3A23", t.url_path())

    def test_create_start_using_string_date(self):
        """We can pass a string (iso8601) instance as the start argument"""
        t = self.create_request_object(dataset_type="raw", formoid="DM", start="2012-12-01")
        self.assertEqual("Mediflex", t.project_name)
        self.assertEqual("Prod", t.environment_name)
        self.assertEqual("studies/Mediflex(Prod)/datasets/raw/DM?start=2012-12-01", t.url_path())


class TestPostDataRequest(unittest.TestCase):
    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""    <Response ReferenceNumber="82e942b0-48e8-4cf4-b299-51e2b6a89a1b"
              InboundODMFileOID=""
              IsTransactionSuccessful="1"
              SuccessStatistics="Rave objects touched: Subjects=0; Folders=0; Forms=0; Fields=0; LogLines=0" NewRecords=""
              SubjectNumberInStudy="1103" SubjectNumberInStudySite="55">
        </Response>"""
        t = PostDataRequest("""some data""")
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSPostResponse))
        self.assertTrue(55, response.subjects_in_study_site)
        self.assertTrue(1103, response.subjects_in_study)


class TestPostMetadataRequest(unittest.TestCase):
    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""    <Response ReferenceNumber="82e942b0-48e8-4cf4-b299-51e2b6a89a1b"
              InboundODMFileOID=""
              IsTransactionSuccessful="1"
              SuccessStatistics="Rave objects touched: Subjects=0; Folders=0; Forms=0; Fields=0; LogLines=0" NewRecords=""
              SubjectNumberInStudy="1103" SubjectNumberInStudySite="55">
        </Response>"""
        t = PostMetadataRequest("Fixitol(Dev)", "Some Data")
        self.assertEqual("Fixitol(Dev)", t.project_name)
        self.assertEqual('metadata/studies/Fixitol(Dev)/drafts', t.url_path())
        self.assertEqual({'Content-type': 'text/xml'}, t.args().get('headers'))
        self.assertEqual('Some Data', t.args().get('data'))
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSPostResponse))
        self.assertTrue(55, response.subjects_in_study_site)
        self.assertTrue(1103, response.subjects_in_study)


class TestGlobalLibraryVersionRequest(unittest.TestCase):
    def create_request_object(self, **kwargs):
        default = dict(project_name="Fixitol(Dev)", oid="1234")
        default.update(kwargs)
        return GlobalLibraryVersionRequest(**default)

    def test_computed_url(self):
        """We get the correct URL for GlobalLibraryVersionRequest"""
        t = self.create_request_object()
        self.assertEqual("metadata/libraries/Fixitol(Dev)/versions/1234", t.url_path())

    def test_checks_for_numeric_oid(self):
        """We get the correct URL for GlobalLibraryVersionRequest"""
        with self.assertRaises(ValueError) as exc:
            t = self.create_request_object(oid='Study1234')
        self.assertEqual('oid must be an integer', str(exc.exception))

class TestGlobalLibraryVersionsRequest(unittest.TestCase):
    def create_request_object(self, project_name="Fixitol(Dev)"):
        t = GlobalLibraryVersionsRequest(project_name=project_name)
        return t

    def test_computed_url(self):
        t = self.create_request_object()
        self.assertEqual("metadata/libraries/Fixitol(Dev)/versions", t.url_path())

    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""<ODM ODMVersion="1.3" Granularity="Metadata" FileType="Snapshot" FileOID="d26b4d33-376d-4037-9747-684411190179" CreationDateTime=" 2013-04-08T01:29:13 " xmlns="http://www.cdisc.org/ns/odm/v1.3" xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata">
        <Study OID="IANTEST">
            <GlobalVariables>
                <StudyName>IANTEST</StudyName>
                <StudyDescription></StudyDescription>
                <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
            <MetaDataVersion OID="1203" Name="Webservice Outbound" />
            <MetaDataVersion OID="1195" Name="JC_Demo_Draft1" />
            <MetaDataVersion OID="1165" Name="Initial" />
        </Study>
    </ODM>
        """
        t = self.create_request_object()
        self.assertEqual("Fixitol(Dev)", t.project_name)
        self.assertEqual('metadata/libraries/Fixitol(Dev)/versions', t.url_path())
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSStudyMetadataVersions))
        for mdv in response:
            self.assertTrue(mdv.oid in ['1203', '1195', '1165'])
            self.assertTrue(mdv.name in ['Webservice Outbound', 'JC_Demo_Draft1','Initial'])


class TestGlobalLibraryDraftsRequest(unittest.TestCase):
    def create_request_object(self, project_name="Fixitol(Dev)"):
        t = GlobalLibraryDraftsRequest(project_name=project_name)
        return t

    def test_computed_url(self):
        t = self.create_request_object()
        self.assertEqual("metadata/libraries/Fixitol(Dev)/drafts", t.url_path())

    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""<ODM ODMVersion="1.3" Granularity="Metadata" FileType="Snapshot" FileOID="d26b4d33-376d-4037-9747-684411190179" CreationDateTime=" 2013-04-08T01:29:13 " xmlns="http://www.cdisc.org/ns/odm/v1.3" xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata">
        <Study OID="IANTEST">
            <GlobalVariables>
                <StudyName>IANTEST</StudyName>
                <StudyDescription></StudyDescription>
                <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
            <MetaDataVersion OID="1203" Name="Webservice Outbound" />
            <MetaDataVersion OID="1195" Name="JC_Demo_Draft1" />
            <MetaDataVersion OID="1165" Name="Initial" />
        </Study>
    </ODM>
        """
        t = self.create_request_object()
        self.assertEqual("Fixitol(Dev)", t.project_name)
        self.assertEqual('metadata/libraries/Fixitol(Dev)/drafts', t.url_path())
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSStudyMetadataVersions))
        for mdv in response:
            self.assertTrue(mdv.oid in ['1203', '1195', '1165'])
            self.assertTrue(mdv.name in ['Webservice Outbound', 'JC_Demo_Draft1','Initial'])


class TestGlobalLibrariesRequest(unittest.TestCase):
    def create_request_object(self, project_name="Fixitol(Dev)"):
        t = GlobalLibrariesRequest()
        return t

    def test_computed_url(self):
        t = self.create_request_object()
        self.assertEqual("metadata/libraries", t.url_path())

    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""    <ODM FileType="Snapshot" FileOID="767a1f8b-7b72-4d12-adbe-37d4d62ba75e"
         CreationDateTime="2013-04-08T10:02:17.781-00:00"
         ODMVersion="1.3"
         xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata"
         xmlns:xlink="http://www.w3.org/1999/xlink"
         xmlns="http://www.cdisc.org/ns/odm/v1.3">
         <Study OID="Fixitol(Dev)">
            <GlobalVariables>
                  <StudyName>Fixitol(Dev)</StudyName>
                  <StudyDescription/>
                  <ProtocolName>Fixitol</ProtocolName>
            </GlobalVariables>
         </Study>
         <Study OID="IANTEST(Prod)">
            <GlobalVariables>
                  <StudyName>IANTEST(Prod)</StudyName>
                  <StudyDescription/>
                  <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
         </Study>
    </ODM>

        """
        t = self.create_request_object()
        self.assertEqual('metadata/libraries', t.url_path())
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSStudies))
        for study in response:
            self.assertTrue(study.oid in ["IANTEST(Prod)", "Fixitol(Dev)"])
            self.assertTrue(study.environment in ["Prod", "Dev"],
                            "Unexpected environment (%s)" % study.environment)


class TestStudyVersionRequest(unittest.TestCase):

    def create_request_object(self, project_name="Fixitol(Dev)", oid="1234"):
        t = StudyVersionRequest(project_name=project_name, oid=oid)
        return t

    def test_computed_url(self):
        t = self.create_request_object()
        self.assertEqual('1234', t.oid)
        self.assertEqual("metadata/studies/Fixitol(Dev)/versions/1234", t.url_path())

    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""    <ODM FileType="Snapshot" FileOID="767a1f8b-7b72-4d12-adbe-37d4d62ba75e"
         CreationDateTime="2013-04-08T10:02:17.781-00:00"
         ODMVersion="1.3"
         xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata"
         xmlns:xlink="http://www.w3.org/1999/xlink"
         xmlns="http://www.cdisc.org/ns/odm/v1.3">
         <Study OID="Fixitol(Dev)">
            <GlobalVariables>
                  <StudyName>Fixitol(Dev)</StudyName>
                  <StudyDescription/>
                  <ProtocolName>Fixitol</ProtocolName>
            </GlobalVariables>
         </Study>
         <Study OID="IANTEST(Prod)">
            <GlobalVariables>
                  <StudyName>IANTEST(Prod)</StudyName>
                  <StudyDescription/>
                  <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
         </Study>
    </ODM>
        """
        t = self.create_request_object()
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertEqual(response_content, response)


class TestStudyVersionsRequest(unittest.TestCase):
    def create_request_object(self, project_name="Fixitol(Dev)"):
        t = StudyVersionsRequest(project_name=project_name)
        return t

    def test_computed_url(self):
        t = self.create_request_object()
        self.assertEqual("metadata/studies/Fixitol(Dev)/versions", t.url_path())

    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""<ODM ODMVersion="1.3" Granularity="Metadata" FileType="Snapshot" FileOID="d26b4d33-376d-4037-9747-684411190179" CreationDateTime=" 2013-04-08T01:29:13 " xmlns="http://www.cdisc.org/ns/odm/v1.3" xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata">
        <Study OID="IANTEST">
            <GlobalVariables>
                <StudyName>IANTEST</StudyName>
                <StudyDescription></StudyDescription>
                <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
            <MetaDataVersion OID="1203" Name="Webservice Outbound" />
            <MetaDataVersion OID="1195" Name="JC_Demo_Draft1" />
            <MetaDataVersion OID="1165" Name="Initial" />
        </Study>
    </ODM>
        """
        t = self.create_request_object()
        self.assertEqual("Fixitol(Dev)", t.project_name)
        self.assertEqual('metadata/studies/Fixitol(Dev)/versions', t.url_path())
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSStudyMetadataVersions))
        for mdv in response:
            self.assertTrue(mdv.oid in ['1203', '1195', '1165'])
            self.assertTrue(mdv.name in ['Webservice Outbound', 'JC_Demo_Draft1','Initial'])


class TestStudyDraftsRequest(unittest.TestCase):
    def create_request_object(self, project_name="Fixitol(Dev)"):
        t = StudyDraftsRequest(project_name=project_name)
        return t

    def test_computed_url(self):
        t = self.create_request_object()
        self.assertEqual("metadata/studies/Fixitol(Dev)/drafts", t.url_path())

    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""<ODM ODMVersion="1.3" Granularity="Metadata" FileType="Snapshot" FileOID="d26b4d33-376d-4037-9747-684411190179" CreationDateTime=" 2013-04-08T01:29:13 " xmlns="http://www.cdisc.org/ns/odm/v1.3" xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata">
        <Study OID="IANTEST">
            <GlobalVariables>
                <StudyName>IANTEST</StudyName>
                <StudyDescription></StudyDescription>
                <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
            <MetaDataVersion OID="1203" Name="Webservice Outbound" />
            <MetaDataVersion OID="1195" Name="JC_Demo_Draft1" />
            <MetaDataVersion OID="1165" Name="Initial" />
        </Study>
    </ODM>
        """
        t = self.create_request_object()
        self.assertEqual("Fixitol(Dev)", t.project_name)
        self.assertEqual('metadata/studies/Fixitol(Dev)/drafts', t.url_path())
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSStudyMetadataVersions))
        for mdv in response:
            self.assertTrue(mdv.oid in ['1203', '1195', '1165'])
            self.assertTrue(mdv.name in ['Webservice Outbound', 'JC_Demo_Draft1','Initial'])


class TestMetadataStudiesRequest(unittest.TestCase):
    def create_request_object(self, project_name="Fixitol(Dev)"):
        t = MetadataStudiesRequest()
        return t

    def test_computed_url(self):
        t = self.create_request_object()
        self.assertEqual("metadata/studies", t.url_path())

    def test_post_data_request_response(self):
        """We can POST data and reformat the response"""
        response_content = u"""    <ODM FileType="Snapshot" FileOID="767a1f8b-7b72-4d12-adbe-37d4d62ba75e"
         CreationDateTime="2013-04-08T10:02:17.781-00:00"
         ODMVersion="1.3"
         xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata"
         xmlns:xlink="http://www.w3.org/1999/xlink"
         xmlns="http://www.cdisc.org/ns/odm/v1.3">
         <Study OID="Fixitol(Dev)">
            <GlobalVariables>
                  <StudyName>Fixitol(Dev)</StudyName>
                  <StudyDescription/>
                  <ProtocolName>Fixitol</ProtocolName>
            </GlobalVariables>
         </Study>
         <Study OID="IANTEST(Prod)">
            <GlobalVariables>
                  <StudyName>IANTEST(Prod)</StudyName>
                  <StudyDescription/>
                  <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
         </Study>
    </ODM>

        """
        t = self.create_request_object()
        self.assertEqual('metadata/studies', t.url_path())
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSStudies))
        for study in response:
            self.assertTrue(study.oid in ["IANTEST(Prod)", "Fixitol(Dev)"])
            self.assertTrue(study.environment in ["Prod", "Dev"],
                            "Unexpected environment (%s)" % study.environment)


class TestClinicalStudiesRequest(unittest.TestCase):

    def create_request_object(self):
        t = ClinicalStudiesRequest()
        return t

    def test_computed_url(self):
        """We evaluate the path for ClinicalStudiesRequest"""
        t = self.create_request_object()
        self.assertEqual("studies", t.url_path())

    def test_process_response(self):
        """We process the response"""
        t = self.create_request_object()
        response_content = u"""<ODM FileType="Snapshot" FileOID="" CreationDateTime="" ODMVersion="1.3"
xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata" xmlns="http://www.cdisc.org/ns/odm/v1.3">
 <Study OID="Lab Test">
   <GlobalVariables>
      <StudyName>Lab Test</StudyName>
      <StudyDescription />
      <ProtocolName>Lab Test</ProtocolName>
   </GlobalVariables>
 </Study>
 <Study OID="Mediflex">
   <GlobalVariables>
      <StudyName>Mediflex</StudyName>
      <StudyDescription />
      <ProtocolName>Mediflex</ProtocolName>
   </GlobalVariables>
 </Study>
</ODM>"""
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSStudies))
        for study in response:
            self.assertTrue(study.oid in ['Lab Test', 'Mediflex'])


class TestCacheFlushRequest(unittest.TestCase):
    def create_request_object(self):
        t = CacheFlushRequest()
        return t

    def test_computed_url(self):
        """We evaluate the path for ClinicalStudiesRequest"""
        t = self.create_request_object()
        self.assertEqual("webservice.aspx?CacheFlush", t.url_path())

    def test_process_response(self):
        """We process the response"""
        t = self.create_request_object()
        response_content = u"""    <Response ReferenceNumber="82e942b0-48e8-4cf4-b299-51e2b6a89a1b"
        InboundODMFileOID=""
        IsTransactionSuccessful="1"
        SuccessStatistics="Rave objects touched: Subjects=0; Folders=0; Forms=0; Fields=0; LogLines=0" NewRecords="">
    </Response>
    """
        req = mock.Mock(requests.Request, text=response_content)
        response = t.result(req)
        self.assertTrue(isinstance(response, RWSResponse))


class TestDiagnosticsRequest(unittest.TestCase):

    def test_computed_url(self):
        """We evaluate the path for DiagnosticsRequest"""
        t = DiagnosticsRequest()
        self.assertEqual("diagnostics", t.url_path())


class TestBuildVersionRequest(unittest.TestCase):

    def test_computed_url(self):
        """We evaluate the path for BuildVersionRequest"""
        t = BuildVersionRequest()
        self.assertEqual("version/build", t.url_path())


class TestCodeNameRequest(unittest.TestCase):
    def test_computed_url(self):
        """We evaluate the path for CodeNameVersionRequest"""
        t = CodeNameRequest()
        self.assertEqual("version/codename", t.url_path())

class TestTwoHundredRequest(unittest.TestCase):
    def test_computed_url(self):
        """We evaluate the path for TwoHundredRequest"""
        t = TwoHundredRequest()
        self.assertEqual("twohundred", t.url_path())


class TestTimeout(unittest.TestCase):
    """
    Strictly belongs in test_rwslib but it interacts with HttPretty which is used in that unit
    """

    def test_connect_timeout(self):
        """Test against an external website to verify connect timeout"""
        rave = RWSConnection('https://innovate.mdsol.com')
        with mock.patch("requests.sessions.Session.get") as mock_get:
            mock_get.side_effect = requests.exceptions.ConnectTimeout()
            with self.assertRaises(RWSException) as exc:
                rave.send_request(rws_requests.ClinicalStudiesRequest(), verify=False,retries=0)
        self.assertEqual('Server Connection Timeout', str(exc.exception))

    def test_read_timeout(self):
        """Test against an external website to verify read timeout"""
        rave = RWSConnection('https://innovate.mdsol.com')
        with mock.patch("requests.sessions.Session.get") as mock_get:
            mock_get.side_effect = requests.exceptions.ReadTimeout()
            with self.assertRaises(RWSException) as exc:
                rave.send_request(rws_requests.ClinicalStudiesRequest(), verify=False,retries=0)
        self.assertEqual('Server Read Timeout', str(exc.exception))

    def test_no_timeout_unauth(self):
        """No timeout raised, then we get the Authorisation Exception """
        rave = RWSConnection('https://innovate.mdsol.com')
        with mock.patch("requests.sessions.Session.get") as mock_get:
            mock_get.return_value = mock.MagicMock(requests.models.Response, status_code=401,
                                                   headers={}, text='Authorization Header not provided')
            with self.assertRaises(AuthorizationException) as exc:
                rave.send_request(rws_requests.ClinicalStudiesRequest(), timeout=3600, verify=False, retries=0)
        self.assertEqual('Authorization Header not provided', str(exc.exception))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
__author__ = "isparks"

import unittest

from rwslib import rwsobjects


class TestParse(unittest.TestCase):
    def test_parse_with_bom(self):
        """Test parser can throw away BOM"""
        text = u"""\xef\xbb\xbf<?xml version="1.0" encoding="utf-8"?><ODM/>"""
        self.assertEqual("ODM", rwsobjects.parseXMLString(text).tag)

    def test_parse_without_bom(self):
        """Test parser can throw away BOM"""
        text = u"""<?xml version="1.0" encoding="utf-8"?><ODM/>"""
        self.assertEqual("ODM", rwsobjects.parseXMLString(text).tag)

    def test_parse_empty_string(self):
        text = u""
        self.assertEqual(u"", rwsobjects.parseXMLString(text))


class TestParseEnvironment(unittest.TestCase):
    """Test for extraction of environment"""

    def test_simple(self):
        env = rwsobjects.getEnvironmentFromNameAndProtocol("TEST (DEV)", "TEST")
        self.assertEqual(env, "DEV")

    def test_braces_in_name(self):
        env = rwsobjects.getEnvironmentFromNameAndProtocol("TEST (1) (ZZZ)", "TEST (1)")
        self.assertEqual(env, "ZZZ")

    def test_single_right_brace_in_name(self):
        env = rwsobjects.getEnvironmentFromNameAndProtocol("TEST :) (PROD)", "TEST :)")
        self.assertEqual(env, "PROD")

    def test_single_left_brace_in_name(self):
        env = rwsobjects.getEnvironmentFromNameAndProtocol("TEST :( (PROD)", "TEST :(")
        self.assertEqual(env, "PROD")

    def test_braces_tight_spaces(self):
        env = rwsobjects.getEnvironmentFromNameAndProtocol("TEST(99)(AUX)", "TEST(99)")
        self.assertEqual(env, "AUX")

    def test_no_env(self):
        """Note. This is probably not wanted behaviour but I am documenting here!
           What this is saying is that the study name is TEST(99) and there is no
           environment supplied.
        """
        env = rwsobjects.getEnvironmentFromNameAndProtocol("TEST(99)", "TEST(99)")
        self.assertEqual(env, "")


class TestRWSErrorResponse(unittest.TestCase):
    """Test that RWSErrorResponse correctly reads an error response"""

    def test_parse(self):
        text = u"""<Response ReferenceNumber="0b47fe86-542f-4070-9e7d-16396a5ef08a"
    InboundODMFileOID="Not Supplied"
    IsTransactionSuccessful="0"
    ReasonCode="RWS00092"
    ErrorClientResponseMessage="CRF version not found">
    </Response>"""

        resp = rwsobjects.RWSErrorResponse(text)

        self.assertEqual(False, resp.istransactionsuccessful)
        self.assertEqual("CRF version not found", resp.errordescription)
        self.assertEqual("Not Supplied", resp.inboundodmfileoid)
        self.assertEqual("RWS00092", resp.reasoncode)
        self.assertEqual("0b47fe86-542f-4070-9e7d-16396a5ef08a", resp.referencenumber)


class TestRWSError(unittest.TestCase):
    """Test that RWSError correctly reads an error response"""

    def test_parse(self):
        text = u"""<?xml version="1.0" encoding="utf-8"?>
         <ODM xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata"
         FileType="Snapshot"
         CreationDateTime="2013-04-08T10:28:49.578-00:00"
         FileOID="4d13722a-ceb6-4419-a917-b6ad5d0bc30e"
         ODMVersion="1.3"
         mdsol:ErrorDescription="Incorrect login and password combination. [RWS00008]"
         xmlns="http://www.cdisc.org/ns/odm/v1.3" />"""

        err = rwsobjects.RWSError(text)

        self.assertEqual("1.3", err.ODMVersion)
        self.assertEqual("Snapshot", err.filetype)
        self.assertEqual("4d13722a-ceb6-4419-a917-b6ad5d0bc30e", err.fileoid)
        self.assertEqual(
            "Incorrect login and password combination. [RWS00008]", err.errordescription
        )
        self.assertEqual("2013-04-08T10:28:49.578-00:00", err.creationdatetime)


class TestRWSResponse(unittest.TestCase):
    """Test that RWSResponse correctly reads an RWS response"""

    def test_parse(self):
        text = u"""<Response ReferenceNumber="82e942b0-48e8-4cf4-b299-51e2b6a89a1b"
              InboundODMFileOID=""
              IsTransactionSuccessful="1"
              SuccessStatistics="Rave objects touched: Subjects=1; Folders=2; Forms=3; Fields=4; LogLines=5" NewRecords="">
    </Response>"""

        response = rwsobjects.RWSResponse(text)

        self.assertEqual(
            "82e942b0-48e8-4cf4-b299-51e2b6a89a1b", response.referencenumber
        )
        self.assertEqual(True, response.istransactionsuccessful)
        self.assertEqual(1, response.subjects_touched)
        self.assertEqual(2, response.folders_touched)
        self.assertEqual(3, response.forms_touched)
        self.assertEqual(4, response.fields_touched)
        self.assertEqual(5, response.loglines_touched)


class TestRWSSubjects(unittest.TestCase):
    """Test RWSSubjects"""

    def test_parse(self):
        """
        Parse a simple response
        """
        text = u"""<ODM xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.cdisc.org/ns/odm/v1.3" FileType="Snapshot" FileOID="0d2dcb32-16ca-4ab9-9917-c4b3eef2fb4a" CreationDateTime="2013-09-10T09:33:07.808-00:00" ODMVersion="1.3">
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="1" mdsol:Overdue="No" mdsol:Touched="Yes" mdsol:Empty="No" mdsol:Incomplete="No" mdsol:NonConformant="No" mdsol:RequiresSecondPass="No" mdsol:RequiresReconciliation="No" mdsol:RequiresVerification="No" mdsol:Verified="No" mdsol:Frozen="No" mdsol:Locked="No" mdsol:RequiresReview="No" mdsol:PendingReview="No" mdsol:Reviewed="No" mdsol:RequiresAnswerQuery="No" mdsol:RequiresPendingCloseQuery="No" mdsol:RequiresCloseQuery="No" mdsol:StickyPlaced="No" mdsol:Signed="No" mdsol:SignatureCurrent="No" mdsol:RequiresTranslation="No" mdsol:RequiresCoding="No" mdsol:RequiresPendingAnswerQuery="No" mdsol:RequiresSignature="No" mdsol:ReadyForFreeze="No" mdsol:ReadyForLock="Yes">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="2">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="3" mdsol:Overdue="No" mdsol:Touched="Yes" mdsol:Empty="No" mdsol:Incomplete="Yes" mdsol:NonConformant="No" mdsol:RequiresSecondPass="No" mdsol:RequiresReconciliation="No" mdsol:RequiresVerification="No" mdsol:Verified="No" mdsol:Frozen="No" mdsol:Locked="No" mdsol:RequiresReview="No" mdsol:PendingReview="No" mdsol:Reviewed="No" mdsol:RequiresAnswerQuery="No" mdsol:RequiresPendingCloseQuery="No" mdsol:RequiresCloseQuery="No" mdsol:StickyPlaced="No" mdsol:Signed="No" mdsol:SignatureCurrent="No" mdsol:RequiresTranslation="No" mdsol:RequiresCoding="No" mdsol:RequiresPendingAnswerQuery="No" mdsol:RequiresSignature="No" mdsol:ReadyForFreeze="No" mdsol:ReadyForLock="Yes">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
</ODM>"""

        subjects = rwsobjects.RWSSubjects(text)

        self.assertEqual("0d2dcb32-16ca-4ab9-9917-c4b3eef2fb4a", subjects.fileoid)
        self.assertEqual(3, len(subjects))
        self.assertEqual(True, subjects[0].touched)
        self.assertEqual(False, subjects[0].overdue)
        self.assertEqual(
            None, subjects[1].overdue
        )  # Example where status was not asked for.
        self.assertEqual(True, subjects[2].incomplete)
        self.assertEqual(text, str(subjects))

    def test_parse_no_uuid(self):
        """
        subject_name works when there is no SubjectKeyType
        """
        text = u"""<ODM xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.cdisc.org/ns/odm/v1.3" FileType="Snapshot" FileOID="0d2dcb32-16ca-4ab9-9917-c4b3eef2fb4a" CreationDateTime="2013-09-10T09:33:07.808-00:00" ODMVersion="1.3">
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="1" mdsol:Overdue="No" mdsol:Touched="Yes" mdsol:Empty="No" mdsol:Incomplete="No" mdsol:NonConformant="No" mdsol:RequiresSecondPass="No" mdsol:RequiresReconciliation="No" mdsol:RequiresVerification="No" mdsol:Verified="No" mdsol:Frozen="No" mdsol:Locked="No" mdsol:RequiresReview="No" mdsol:PendingReview="No" mdsol:Reviewed="No" mdsol:RequiresAnswerQuery="No" mdsol:RequiresPendingCloseQuery="No" mdsol:RequiresCloseQuery="No" mdsol:StickyPlaced="No" mdsol:Signed="No" mdsol:SignatureCurrent="No" mdsol:RequiresTranslation="No" mdsol:RequiresCoding="No" mdsol:RequiresPendingAnswerQuery="No" mdsol:RequiresSignature="No" mdsol:ReadyForFreeze="No" mdsol:ReadyForLock="Yes">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="2">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="3" mdsol:Overdue="No" mdsol:Touched="Yes" mdsol:Empty="No" mdsol:Incomplete="Yes" mdsol:NonConformant="No" mdsol:RequiresSecondPass="No" mdsol:RequiresReconciliation="No" mdsol:RequiresVerification="No" mdsol:Verified="No" mdsol:Frozen="No" mdsol:Locked="No" mdsol:RequiresReview="No" mdsol:PendingReview="No" mdsol:Reviewed="No" mdsol:RequiresAnswerQuery="No" mdsol:RequiresPendingCloseQuery="No" mdsol:RequiresCloseQuery="No" mdsol:StickyPlaced="No" mdsol:Signed="No" mdsol:SignatureCurrent="No" mdsol:RequiresTranslation="No" mdsol:RequiresCoding="No" mdsol:RequiresPendingAnswerQuery="No" mdsol:RequiresSignature="No" mdsol:ReadyForFreeze="No" mdsol:ReadyForLock="Yes">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
</ODM>"""

        subjects = rwsobjects.RWSSubjects(text)

        self.assertEqual("0d2dcb32-16ca-4ab9-9917-c4b3eef2fb4a", subjects.fileoid)
        self.assertEqual(3, len(subjects))
        self.assertEqual(True, subjects[0].touched)
        self.assertEqual(False, subjects[0].overdue)
        self.assertEqual(
            None, subjects[1].overdue
        )  # Example where status was not asked for.
        self.assertEqual(
            0, len(subjects[1].links)
        )  # Example where link was not asked for
        self.assertEqual(True, subjects[2].incomplete)
        self.assertEqual(text, str(subjects))
        self.assertEqual("1", subjects[0].subject_name)
        self.assertEqual("2", subjects[1].subject_name)
        self.assertEqual("3", subjects[2].subject_name)

    def test_parse_out_subject_key_where_uuid(self):
        """
        when there is a SubjectKeyType='SubjectUUID' then we return Subject ID consistently
        """
        text = u"""<ODM xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.cdisc.org/ns/odm/v1.3" FileType="Snapshot" FileOID="0d2dcb32-16ca-4ab9-9917-c4b3eef2fb4a" CreationDateTime="2013-09-10T09:33:07.808-00:00" ODMVersion="1.3">
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="0A663F39" mdsol:SubjectName="1" mdsol:SubjectKeyType="SubjectUUID" mdsol:Overdue="No" mdsol:Touched="Yes" mdsol:Empty="No" mdsol:Incomplete="No" mdsol:NonConformant="No" mdsol:RequiresSecondPass="No" mdsol:RequiresReconciliation="No" mdsol:RequiresVerification="No" mdsol:Verified="No" mdsol:Frozen="No" mdsol:Locked="No" mdsol:RequiresReview="No" mdsol:PendingReview="No" mdsol:Reviewed="No" mdsol:RequiresAnswerQuery="No" mdsol:RequiresPendingCloseQuery="No" mdsol:RequiresCloseQuery="No" mdsol:StickyPlaced="No" mdsol:Signed="No" mdsol:SignatureCurrent="No" mdsol:RequiresTranslation="No" mdsol:RequiresCoding="No" mdsol:RequiresPendingAnswerQuery="No" mdsol:RequiresSignature="No" mdsol:ReadyForFreeze="No" mdsol:ReadyForLock="Yes">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="0076F9FE" mdsol:SubjectName="2" mdsol:SubjectKeyType="SubjectUUID">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="B8CFE69E" mdsol:SubjectName="3" mdsol:SubjectKeyType="SubjectUUID" mdsol:Overdue="No" mdsol:Touched="Yes" mdsol:Empty="No" mdsol:Incomplete="Yes" mdsol:NonConformant="No" mdsol:RequiresSecondPass="No" mdsol:RequiresReconciliation="No" mdsol:RequiresVerification="No" mdsol:Verified="No" mdsol:Frozen="No" mdsol:Locked="No" mdsol:RequiresReview="No" mdsol:PendingReview="No" mdsol:Reviewed="No" mdsol:RequiresAnswerQuery="No" mdsol:RequiresPendingCloseQuery="No" mdsol:RequiresCloseQuery="No" mdsol:StickyPlaced="No" mdsol:Signed="No" mdsol:SignatureCurrent="No" mdsol:RequiresTranslation="No" mdsol:RequiresCoding="No" mdsol:RequiresPendingAnswerQuery="No" mdsol:RequiresSignature="No" mdsol:ReadyForFreeze="No" mdsol:ReadyForLock="Yes">
      <SiteRef LocationOID="TESTSITE"/>
    </SubjectData>
  </ClinicalData>
</ODM>"""

        subjects = rwsobjects.RWSSubjects(text)

        self.assertEqual("0d2dcb32-16ca-4ab9-9917-c4b3eef2fb4a", subjects.fileoid)
        self.assertEqual(3, len(subjects))
        self.assertEqual(True, subjects[0].touched)
        self.assertEqual(False, subjects[0].overdue)
        self.assertEqual(
            None, subjects[1].overdue
        )  # Example where status was not asked for.
        self.assertEqual(True, subjects[2].incomplete)
        self.assertEqual(text, str(subjects))
        self.assertEqual("1", subjects[0].subject_name)
        self.assertEqual("2", subjects[1].subject_name)
        self.assertEqual("3", subjects[2].subject_name)
        self.assertEqual("0A663F39", subjects[0].subjectkey)
        self.assertEqual("0076F9FE", subjects[1].subjectkey)
        self.assertEqual("B8CFE69E", subjects[2].subjectkey)

    def test_parse_with_links(self):
        """
        Populate subject.link with URL when provided
        """
        text = u"""<ODM xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns="http://www.cdisc.org/ns/odm/v1.3" FileType="Snapshot" FileOID="0d2dcb32-16ca-4ab9-9917-c4b3eef2fb4a" CreationDateTime="2013-09-10T09:33:07.808-00:00" ODMVersion="1.3">
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="0A663F39" mdsol:SubjectName="1" mdsol:SubjectKeyType="SubjectUUID" mdsol:Overdue="No" mdsol:Touched="Yes" mdsol:Empty="No" mdsol:Incomplete="No" mdsol:NonConformant="No" mdsol:RequiresSecondPass="No" mdsol:RequiresReconciliation="No" mdsol:RequiresVerification="No" mdsol:Verified="No" mdsol:Frozen="No" mdsol:Locked="No" mdsol:RequiresReview="No" mdsol:PendingReview="No" mdsol:Reviewed="No" mdsol:RequiresAnswerQuery="No" mdsol:RequiresPendingCloseQuery="No" mdsol:RequiresCloseQuery="No" mdsol:StickyPlaced="No" mdsol:Signed="No" mdsol:SignatureCurrent="No" mdsol:RequiresTranslation="No" mdsol:RequiresCoding="No" mdsol:RequiresPendingAnswerQuery="No" mdsol:RequiresSignature="No" mdsol:ReadyForFreeze="No" mdsol:ReadyForLock="Yes">
      <SiteRef LocationOID="TESTSITE"/>
      <mdsol:Link xlink:type="simple" xlink:href="http://innovate.mdsol.com/MedidataRAVE/HandleLink.aspx?page=SubjectPage.aspx?ID=1234" />
    </SubjectData>
  </ClinicalData>
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="0076F9FE" mdsol:SubjectName="2" mdsol:SubjectKeyType="SubjectUUID">
      <SiteRef LocationOID="TESTSITE"/>
      <mdsol:Link xlink:type="simple" xlink:href="http://innovate.mdsol.com/MedidataRAVE/HandleLink.aspx?page=SubjectPage.aspx?ID=5678" />
    </SubjectData>
  </ClinicalData>
  <ClinicalData StudyOID="SIMPLESTUDY(TEST)" MetaDataVersionOID="1128">
    <SubjectData SubjectKey="B8CFE69E" mdsol:SubjectName="3" mdsol:SubjectKeyType="SubjectUUID" mdsol:Overdue="No" mdsol:Touched="Yes" mdsol:Empty="No" mdsol:Incomplete="Yes" mdsol:NonConformant="No" mdsol:RequiresSecondPass="No" mdsol:RequiresReconciliation="No" mdsol:RequiresVerification="No" mdsol:Verified="No" mdsol:Frozen="No" mdsol:Locked="No" mdsol:RequiresReview="No" mdsol:PendingReview="No" mdsol:Reviewed="No" mdsol:RequiresAnswerQuery="No" mdsol:RequiresPendingCloseQuery="No" mdsol:RequiresCloseQuery="No" mdsol:StickyPlaced="No" mdsol:Signed="No" mdsol:SignatureCurrent="No" mdsol:RequiresTranslation="No" mdsol:RequiresCoding="No" mdsol:RequiresPendingAnswerQuery="No" mdsol:RequiresSignature="No" mdsol:ReadyForFreeze="No" mdsol:ReadyForLock="Yes">
      <SiteRef LocationOID="TESTSITE"/>
      <mdsol:Link xlink:type="simple" xlink:href="http://innovate.mdsol.com/MedidataRAVE/HandleLink.aspx?page=SubjectPage.aspx?ID=9012" />
    </SubjectData>
  </ClinicalData>
</ODM>"""

        subjects = rwsobjects.RWSSubjects(text)

        self.assertEqual(
            "http://innovate.mdsol.com/MedidataRAVE/HandleLink.aspx?page=SubjectPage.aspx?ID=1234",
            subjects[0].links[0],
        )
        self.assertEqual(
            "http://innovate.mdsol.com/MedidataRAVE/HandleLink.aspx?page=SubjectPage.aspx?ID=5678",
            subjects[1].links[0],
        )
        self.assertEqual(
            "http://innovate.mdsol.com/MedidataRAVE/HandleLink.aspx?page=SubjectPage.aspx?ID=9012",
            subjects[2].links[0],
        )

    def test_site_number(self):
        """
        Gets the Siten
        """
        text = """<?xml version="1.0" encoding="utf-8"?>
<ODM FileType="Snapshot" FileOID="b5cdc2fb-36fd-4cf0-abc7-c84628a7380f" CreationDateTime="2022-06-01T14:16:52.336-00:00" ODMVersion="1.3"
	xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata"
	xmlns:xlink="http://www.w3.org/1999/xlink"
	xmlns="http://www.cdisc.org/ns/odm/v1.3">
	<ClinicalData StudyOID="TDE-OSTEO-800(Prod)" MetaDataVersionOID="104">
		<SubjectData SubjectKey="6c3eed1c-a256-4c0b-8dff-49ea21e20cc5" mdsol:SubjectKeyType="SubjectUUID" mdsol:SubjectName="0010">
			<SiteRef LocationOID="mdsol03" mdsol:StudyEnvSiteNumber="Site002" />
		</SubjectData>
	</ClinicalData>
	<ClinicalData StudyOID="TDE-OSTEO-800(Prod)" MetaDataVersionOID="109">
		<SubjectData SubjectKey="02e74501-0d1b-4000-a6fb-ffe2e6613875" mdsol:SubjectKeyType="SubjectUUID" mdsol:SubjectName="0011">
			<SiteRef LocationOID="810" mdsol:StudyEnvSiteNumber="site 003" />
		</SubjectData>
	</ClinicalData>
	</ODM>
"""
        subjects = rwsobjects.RWSSubjects(text)
        self.assertEqual(2, len(subjects))
        mapped = {"02e74501-0d1b-4000-a6fb-ffe2e6613875": ["810", "site 003"],
                  "6c3eed1c-a256-4c0b-8dff-49ea21e20cc5": ["mdsol03", "Site002"]}
        for subject in subjects:
            meta = mapped[subject.subjectkey]
            self.assertEqual(meta[1], subject.study_environment_site_number)
            self.assertEqual(meta[0], subject.locationoid)


class TestMetaDataVersions(unittest.TestCase):
    """Test MetaDataVersions"""

    def test_parse(self):
        text = u"""<ODM ODMVersion="1.3" Granularity="Metadata" FileType="Snapshot" FileOID="d26b4d33-376d-4037-9747-684411190179" CreationDateTime=" 2013-04-08T01:29:13 " xmlns="http://www.cdisc.org/ns/odm/v1.3" xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata">
        <Study OID="IANTEST">
            <GlobalVariables>
                <StudyName>IANTEST</StudyName>
                <StudyDescription></StudyDescription>
                <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
            <MetaDataVersion OID="1203" Name="Webservice Outbound" />
            <MetaDataVersion OID="1195" Name="Demo_Draft1" />
            <MetaDataVersion OID="1165" Name="Initial" />
        </Study>
    </ODM>"""

        metadata = rwsobjects.RWSStudyMetadataVersions(text)

        self.assertEqual("1.3", metadata.ODMVersion)
        self.assertEqual("IANTEST", metadata.study.oid)
        self.assertEqual(3, len(metadata))
        self.assertEqual("Demo_Draft1", metadata[1].name)
        self.assertEqual("1165", metadata[2].oid)


class TestRWSPostResponse(unittest.TestCase):
    def test_parse(self):
        text = u"""<Response ReferenceNumber="82e942b0-48e8-4cf4-b299-51e2b6a89a1b"
              InboundODMFileOID=""
              IsTransactionSuccessful="1"
              SuccessStatistics="Rave objects touched: Subjects=0; Folders=0; Forms=0; Fields=0; LogLines=0" NewRecords=""
              SubjectNumberInStudy="1103" SubjectNumberInStudySite="55">
    </Response>"""

        parsed = rwsobjects.RWSPostResponse(text)

        self.assertEqual(True, parsed.istransactionsuccessful)
        self.assertEqual(1103, parsed.subjects_in_study)
        self.assertEqual(55, parsed.subjects_in_study_site)

    def test_bad_success_stats(self):
        """Test error when statistics contain info we don't recognize"""
        text = u"""<Response ReferenceNumber="82e942b0-48e8-4cf4-b299-51e2b6a89a1b"
              InboundODMFileOID=""
              IsTransactionSuccessful="1"
              SuccessStatistics="Rave objects touched: Subjects=0; Folders=0; Forms=0; Unknown=10; Fields=0; LogLines=0"
              NewRecords=""
              SubjectNumberInStudy="1103" SubjectNumberInStudySite="55">
    </Response>"""
        with self.assertRaises(KeyError):
            rwsobjects.RWSPostResponse(text)


class TestRWSPostErrorResponse(unittest.TestCase):
    def test_parse(self):
        text = u"""<Response
        ReferenceNumber="5b1fa9a3-0cf3-46b6-8304-37c2e3b7d04f5"
        InboundODMFileOID="1"
        IsTransactionSuccessful = "0"
        ReasonCode="RWS00024"
        ErrorOriginLocation="/ODM/ClinicalData[1]/SubjectData[1]"
        SuccessStatistics="Rave objects touched: Subjects=0; Folders=0; Forms=0; Fields=0; LogLines=0"
        ErrorClientResponseMessage="Subject already exists.">
        </Response>
        """

        parsed = rwsobjects.RWSPostErrorResponse(text)
        self.assertEqual(
            "5b1fa9a3-0cf3-46b6-8304-37c2e3b7d04f5", parsed.referencenumber
        )
        self.assertEqual(
            "Subject already exists.", parsed.error_client_response_message
        )
        self.assertEqual(
            "/ODM/ClinicalData[1]/SubjectData[1]", parsed.error_origin_location
        )
        self.assertEqual("RWS00024", parsed.reason_code)
        self.assertEqual("RWS00024", parsed.reasoncode)


class TestRWSStudies(unittest.TestCase):
    def test_parse(self):
        text = u"""<ODM FileType="Snapshot" FileOID="767a1f8b-7b72-4d12-adbe-37d4d62ba75e"
         CreationDateTime="2013-04-08T10:02:17.781-00:00"
         ODMVersion="1.3"
         xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata"
         xmlns:xlink="http://www.w3.org/1999/xlink"
         xmlns="http://www.cdisc.org/ns/odm/v1.3">
         <Study OID="Fixitol(Dev)">
            <GlobalVariables>
                  <StudyName>Fixitol (Dev)</StudyName>
                  <StudyDescription/>
                  <ProtocolName>Fixitol</ProtocolName>
            </GlobalVariables>
         </Study>
         <Study OID="IANTEST(Prod)">
            <GlobalVariables>
                  <StudyName>IANTEST</StudyName>
                  <StudyDescription/>
                  <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
         </Study>
    </ODM>"""
        parsed = rwsobjects.RWSStudies(text)
        self.assertEqual("1.3", parsed.ODMVersion)
        self.assertEqual(2, len(parsed))
        self.assertEqual(False, parsed[0].isProd())
        self.assertEqual(True, parsed[1].isProd())


class TestUtils(unittest.TestCase):
    def test_parse(self):
        text = u"""<ODM FileType="Snapshot" FileOID="767a1f8b-7b72-4d12-adbe-37d4d62ba75e"
         CreationDateTime="2013-04-08T10:02:17.781-00:00"
         ODMVersion="1.3"
         xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata"
         xmlns:xlink="http://www.w3.org/1999/xlink"
         xmlns="http://www.cdisc.org/ns/odm/v1.3">
         <Study OID="Fixitol(Dev)">
            <GlobalVariables>
                  <StudyName>Fixitol (Dev)</StudyName>
                  <StudyDescription/>
                  <ProtocolName>Fixitol</ProtocolName>
            </GlobalVariables>
         </Study>
         <Study OID="IANTEST(Prod)">
            <GlobalVariables>
                  <StudyName>IANTEST</StudyName>
                  <StudyDescription/>
                  <ProtocolName>IANTEST</ProtocolName>
            </GlobalVariables>
         </Study>
    </ODM>"""
        parsed = rwsobjects.RWSStudies(text)
        self.assertEqual("1.3", parsed.ODMVersion)
        self.assertEqual(2, len(parsed))
        self.assertEqual(False, parsed[0].isProd())
        self.assertEqual(True, parsed[1].isProd())


if __name__ == "__main__":
    unittest.main()



from rwslib.extras.audit_event import parser
import unittest
import os
import pytest



class MockEventer:
    """
    Mock Event Sink instance for the purposes of testing (capturing all ASC)
    """

    def __init__(self):
        self.__events = {}

    def default(self, event):
        self.__events.setdefault(event.subcategory, []).append(event)

    def get_audit_subcategory_events(self, audit_subcategory):
        return self.__events[audit_subcategory]

    @property
    def eventlist(self):
        return self.__events.keys()


class MockEventerEntered:
    """
    Mock Event Sink instance for the purposes of testing (capturing only 'Entered' ASC)
    """

    def __init__(self):
        self.__events = {}

    def get_audit_subcategory_events(self, audit_subcategory):
        return self.__events.get(audit_subcategory, [])

    def Entered(self, event):
        self.__events.setdefault(event.subcategory, []).append(event)

    @property
    def eventlist(self):
        return self.__events.keys()


@pytest.mark.usefixtures("car_message")
class TestAuditEvent(unittest.TestCase):
    """
    Test Case for Audit Event Processor
    """

    def test_parses_audit_message(self):
        """parses an audit message from a CAR message"""
        eventer = MockEventer()
        message = parser.parse(self.car_message, eventer)
        # get the events
        self.assertTrue(len(eventer.eventlist) > 1)
        self.assertTrue("EnteredEmpty" in eventer.eventlist)
        self.assertEqual(60, len(eventer.get_audit_subcategory_events("EnteredEmpty")))
        self.assertEqual(501, len(eventer.get_audit_subcategory_events("Entered")))

    def test_parses_audit_message_entered(self):
        """parses an audit message, but only subscribe to Entered Events from a CAR message"""
        eventer = MockEventerEntered()
        message = parser.parse(self.car_message, eventer)
        # get the events
        self.assertTrue(len(eventer.eventlist) == 1)
        self.assertTrue("EnteredEmpty" not in eventer.eventlist)
        self.assertEqual(0, len(eventer.get_audit_subcategory_events("EnteredEmpty")))
        self.assertEqual(501, len(eventer.get_audit_subcategory_events("Entered")))


    def test_parses_audit_message_subject_created(self):
        """parses an audit message, but only subscribe to Entered Events from a CAR message"""

        eventer = MockEventer()
        message = parser.parse(self.car_message, eventer)
        # get the events
        self.assertEqual(
            92, len(eventer.get_audit_subcategory_events("SubjectCreated"))
        )
        subject_123_ABC = eventer.get_audit_subcategory_events("SubjectCreated")[0]
        self.assertEqual(
            "e983f330-c108-45ab-8f16-b4a566c7089c", subject_123_ABC.subject.key
        )
        self.assertEqual("123 ABC", subject_123_ABC.subject.name)

    def test_parses_specify_value(self):
        """Extracts a specified value from a CAR message"""
        content = """<ODM ODMVersion="1.3" FileType="Transactional" FileOID="552a8cac-7c4e-4ba5-9f71-a20b90865531" CreationDateTime="2021-06-02T10:21:02" xmlns="http://www.cdisc.org/ns/odm/v1.3" xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata">
            <ClinicalData StudyOID="Mediflex" MetaDataVersionOID="16"  mdsol:AuditSubCategoryName="Entered">
        <SubjectData SubjectKey="e983f330-c108-45ab-8f16-b4a566c7089c" mdsol:SubjectKeyType="SubjectUUID" mdsol:SubjectName="123 ABC"  >
            <SiteRef LocationOID="MDSOL" />
            <StudyEventData StudyEventOID="SCREEN"  StudyEventRepeatKey="SCREEN[1]"   mdsol:InstanceId="50" >
                <FormData FormOID="DM" FormRepeatKey="1"  mdsol:DataPageId="179" >
                    <ItemGroupData ItemGroupOID="DM"   mdsol:RecordId="251" >
                        <ItemData ItemOID="DM.SEX" TransactionType="Upsert"  Value="Specify" mdsol:SpecifyValue="UNDEF"	 >
                            <AuditRecord>
                                <UserRef UserOID="pvummudi"/>
                                <LocationRef LocationOID="MDSOL" />
                                <DateTimeStamp>2008-12-04T16:59:07</DateTimeStamp>
                                <ReasonForChange></ReasonForChange>
                                <SourceID>3284</SourceID>
                            </AuditRecord>
                        </ItemData>
                    </ItemGroupData>
                </FormData>
            </StudyEventData>
        </SubjectData>
    </ClinicalData>
        </ODM>
        """
        eventer = MockEventer()
        message = parser.parse(content, eventer)
        event = eventer.get_audit_subcategory_events("Entered")[0]
        self.assertEqual("UNDEF", event.item.specify_value)
        self.assertEqual("Specify", event.item.value)
        self.assertEqual("DM.SEX", event.item.oid)

    def test_parses_signature_broken(self):
        """mdsol:SignatureBroken parsing for case when broken from a CAR message"""
        content = """<ODM ODMVersion="1.3" FileType="Transactional" FileOID="552a8cac-7c4e-4ba5-9f71-a20b90865531" CreationDateTime="2021-06-02T10:21:02" xmlns="http://www.cdisc.org/ns/odm/v1.3" xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata">
            <ClinicalData StudyOID="Mediflex" MetaDataVersionOID="16"  mdsol:AuditSubCategoryName="Entered">
        <SubjectData SubjectKey="e983f330-c108-45ab-8f16-b4a566c7089c" mdsol:SubjectKeyType="SubjectUUID" mdsol:SubjectName="123 ABC"  >
            <SiteRef LocationOID="MDSOL" />
            <StudyEventData StudyEventOID="SCREEN"  StudyEventRepeatKey="SCREEN[1]"   mdsol:InstanceId="50" >
                <FormData FormOID="DM" FormRepeatKey="1"  mdsol:DataPageId="179" >
                    <ItemGroupData ItemGroupOID="DM"   mdsol:RecordId="251" >
                        <ItemData ItemOID="DM.SEX" TransactionType="Upsert"  Value="Specify" mdsol:SpecifyValue="UNDEF"	mdsol:SignatureBroken="Yes" >
                            <AuditRecord>
                                <UserRef UserOID="pvummudi"/>
                                <LocationRef LocationOID="MDSOL" />
                                <DateTimeStamp>2008-12-04T16:59:07</DateTimeStamp>
                                <ReasonForChange></ReasonForChange>
                                <SourceID>3284</SourceID>
                            </AuditRecord>
                        </ItemData>
                    </ItemGroupData>
                </FormData>
            </StudyEventData>
        </SubjectData>
    </ClinicalData>
        </ODM>
        """
        eventer = MockEventer()
        message = parser.parse(content, eventer)
        event = eventer.get_audit_subcategory_events("Entered")[0]
        self.assertEqual("UNDEF", event.item.specify_value)
        self.assertEqual("Specify", event.item.value)
        self.assertEqual("DM.SEX", event.item.oid)
        self.assertTrue(event.item.signature_broken)

    def test_parses_signature_not_broken(self):
        """mdsol:SignatureBroken parsing for case when not broken from a CAR message"""
        content = """<ODM ODMVersion="1.3" FileType="Transactional" FileOID="552a8cac-7c4e-4ba5-9f71-a20b90865531" CreationDateTime="2021-06-02T10:21:02" xmlns="http://www.cdisc.org/ns/odm/v1.3" xmlns:mdsol="http://www.mdsol.com/ns/odm/metadata">
            <ClinicalData StudyOID="Mediflex" MetaDataVersionOID="16"  mdsol:AuditSubCategoryName="Entered">
        <SubjectData SubjectKey="e983f330-c108-45ab-8f16-b4a566c7089c" mdsol:SubjectKeyType="SubjectUUID" mdsol:SubjectName="123 ABC"  >
            <SiteRef LocationOID="MDSOL" />
            <StudyEventData StudyEventOID="SCREEN"  StudyEventRepeatKey="SCREEN[1]"   mdsol:InstanceId="50" >
                <FormData FormOID="DM" FormRepeatKey="1"  mdsol:DataPageId="179" >
                    <ItemGroupData ItemGroupOID="DM"   mdsol:RecordId="251" >
                        <ItemData ItemOID="DM.SEX" TransactionType="Upsert"  Value="Specify" mdsol:SpecifyValue="UNDEF"	mdsol:SignatureBroken="No" >
                            <AuditRecord>
                                <UserRef UserOID="pvummudi"/>
                                <LocationRef LocationOID="MDSOL" />
                                <DateTimeStamp>2008-12-04T16:59:07</DateTimeStamp>
                                <ReasonForChange></ReasonForChange>
                                <SourceID>3284</SourceID>
                            </AuditRecord>
                        </ItemData>
                    </ItemGroupData>
                </FormData>
            </StudyEventData>
        </SubjectData>
    </ClinicalData>
        </ODM>
        """
        eventer = MockEventer()
        message = parser.parse(content, eventer)
        event = eventer.get_audit_subcategory_events("Entered")[0]
        self.assertEqual("UNDEF", event.item.specify_value)
        self.assertEqual("Specify", event.item.value)
        self.assertEqual("DM.SEX", event.item.oid)
        self.assertFalse(event.item.signature_broken)




# -*- coding: utf-8 -*-

__author__ = 'anewbigging'

from rwslib import RWSConnection
from rwslib.rws_requests import *
from rwslib.rwsobjects import RWSException
import requests
from requests.auth import HTTPBasicAuth
from lxml import etree
from functools import partial
from rwslib.extras.rwscmd.odmutils import xml_pretty_print, E_ODM, A_ODM
from rwslib.extras.rwscmd.data_scrambler import Scramble
import click

GET_DATA_DATASET = 'rwscmd_getdata.odm'


class GetDataConfigurableDataset(ConfigurableDatasetRequest):
    VALID_DATASET_FORMATS = ("odm")

    def __init__(self, dataset, study, environment, subject, params=None):
        dataset_name, dataset_format = dataset.split('.')
        studyoid = "{}({})".format(study, environment)
        if params is None:
            params = {}
        params.update(dict(StudyOID=studyoid, SubjectKey=subject))
        super(GetDataConfigurableDataset, self).__init__(dataset_name, dataset_format, params)


@click.group()
@click.option('--username', '-u', prompt=True, default='', envvar='RWSCMD_USERNAME', help='Rave login')
@click.option('--password', '-p', prompt=True, default='', hide_input=True, envvar='RWSCMD_PASSWORD',
              help='Rave password')
@click.option('--virtual_dir', default=None, envvar='RWSCMD_VIRTUAL_DIR',
              help='RWS virtual directory, defaults to RaveWebServices')
@click.option('--raw/--list', default=False,
              help='Display raw xml response from RWS or human-readable list, defaults to list')
@click.option('--verbose/--silent', '-v/-s', default=False)
@click.option('--output', '-o', default=None, type=click.File('wb'), help='Write output to file')
@click.argument('url')
@click.pass_context
def rws(ctx, url, username, password, raw, verbose, output, virtual_dir):
    if ctx.obj is None:
        ctx.obj = {}
    ctx.obj['URL'] = url
    ctx.obj['USERNAME'] = username
    ctx.obj['PASSWORD'] = password
    ctx.obj['VIRTUAL_DIR'] = virtual_dir
    if virtual_dir:
        if username and password:
            ctx.obj['RWS'] = RWSConnection(url, username, password, virtual_dir=virtual_dir)
        else:
            # Acceptable for UnAuth Requests
            ctx.obj['RWS'] = RWSConnection(url, virtual_dir=virtual_dir)
    else:
        if username and password:
            ctx.obj['RWS'] = RWSConnection(url, username, password)
        else:
            ctx.obj['RWS'] = RWSConnection(url)
    ctx.obj['RAW'] = raw
    ctx.obj['OUTPUT'] = output
    ctx.obj['VERBOSE'] = verbose


def get_data(ctx, study, environment, subject):
    """
    Call rwscmd_getdata custom dataset to retrieve currently enterable, empty fields
    """
    cfg = GetDataConfigurableDataset(GET_DATA_DATASET,
                                     study,
                                     environment,
                                     subject,
                                     params=dict(IncludeIDs=0,
                                                 IncludeValues=0))
    # path = "datasets/{}?StudyOID={}&SubjectKey={}" \
    #        "&IncludeIDs=0&IncludeValues=0".format(GET_DATA_DATASET, studyoid, subject)
    # url = make_url(ctx.obj['RWS'].base_url, path)

    if ctx.obj['VERBOSE']:
        click.echo('Getting data list')
    # Get the client instance
    client = ctx.obj['RWS']  #: type: RWSConnection
    # Client rolls in the base_url
    resp = client.send_request(cfg)
    # resp = requests.get(url, auth=HTTPBasicAuth(ctx.obj['USERNAME'], ctx.obj['PASSWORD']))

    if client.last_result.status_code != 200:
        click.echo(client.last_result.text)

    return xml_pretty_print(resp)


def rws_call(ctx, method, default_attr=None):
    """Make request to RWS"""
    try:
        response = ctx.obj['RWS'].send_request(method)

        if ctx.obj['RAW']:  # use response from RWS
            result = ctx.obj['RWS'].last_result.text
        elif default_attr is not None:  # human-readable summary
            result = ""
            for item in response:
                result = result + item.__dict__[default_attr] + "\n"
        else:  # use response from RWS
            result = ctx.obj['RWS'].last_result.text

        if ctx.obj['OUTPUT']:  # write to file
            ctx.obj['OUTPUT'].write(result.encode('utf-8'))
        else:  # echo
            click.echo(result)

    except RWSException as e:
        click.echo(str(e))


@rws.command()
@click.pass_context
def version(ctx):
    """Display RWS version"""
    rws_call(ctx, VersionRequest())


@rws.command()
@click.argument('path', nargs=-1)
@click.pass_context
def data(ctx, path):
    """List EDC data for [STUDY] [ENV] [SUBJECT]"""
    _rws = partial(rws_call, ctx)
    if len(path) == 0:
        _rws(ClinicalStudiesRequest(), default_attr='oid')
    elif len(path) == 1:
        _rws(StudySubjectsRequest(path[0], 'Prod'), default_attr='subjectkey')
    elif len(path) == 2:
        _rws(StudySubjectsRequest(path[0], path[1]), default_attr='subjectkey')
    elif len(path) == 3:
        try:
            click.echo(get_data(ctx, path[0], path[1], path[2]))
        except RWSException as e:
            click.echo(str(e))
        except requests.exceptions.HTTPError as e:
            click.echo(str(e))
    else:
        click.echo('Too many arguments')


@rws.command()
@click.argument('odm', type=click.File('rb'))
@click.pass_context
def post(ctx, odm):
    """Post ODM clinical data"""
    try:
        ctx.obj['RWS'].send_request(PostDataRequest(odm.read()))
        if ctx.obj['RAW']:
            click.echo(ctx.obj['RWS'].last_result.text)
    except RWSException as e:
        click.echo(e.message)


@rws.command()
@click.option('--drafts/--versions', default=False, help='List CRF drafts or versions (default)')
@click.argument('path', nargs=-1)
@click.pass_context
def metadata(ctx, drafts, path):
    """List metadata for [PROJECT] [VERSION]"""
    _rws = partial(rws_call, ctx)
    if len(path) == 0:
        _rws(MetadataStudiesRequest(), default_attr='oid')
    elif len(path) == 1:
        if drafts:
            _rws(StudyDraftsRequest(path[0]), default_attr='oid')
        else:
            _rws(StudyVersionsRequest(path[0]), default_attr='oid')
    elif len(path) == 2:
        _rws(StudyVersionRequest(path[0], path[1]))
    else:
        click.echo('Too many arguments')


@rws.command()
@click.argument('path')
@click.pass_context
def direct(ctx, path):
    """Make direct call to RWS, bypassing rwslib"""
    try:
        url = make_url(ctx.obj['RWS'].base_url, path)
        resp = requests.get(url, auth=HTTPBasicAuth(ctx.obj['USERNAME'], ctx.obj['PASSWORD']))
        click.echo(resp.text)
    except RWSException as e:
        click.echo(e.message)
    except requests.exceptions.HTTPError as e:
        click.echo(e.message)


@rws.command()
@click.option('--steps', type=click.INT, default=10, help='Number of data entry iterations (default=10)')
@click.option('--metadata', default=None, type=click.File('rb'),
              help='Metadata file (optional)')
@click.option('--fixed', default=None, type=click.File('rb'),
              help='File with values to override generated data (one per line in format ItemOID,Value)')
@click.argument('study')
@click.argument('environment')
@click.argument('subject')
@click.pass_context
def autofill(ctx, steps, metadata, fixed, study, environment, subject):
    """Request enterable data for a subject, generate data values and post back to Rave.
    Requires 'rwscmd_getdata' configurable dataset to be installed on the Rave URL."""

    if metadata is not None:  # Read metadata from file, if supplied
        odm_metadata = metadata.read()
        meta_v = etree.fromstring(odm_metadata).find('.//' + E_ODM.METADATA_VERSION.value).get(A_ODM.OID.value)
    else:
        odm_metadata = None
        meta_v = None

    fixed_values = {}
    if fixed is not None:  # Read fixed values from file, if supplied
        for f in fixed:
            oid, value = f.decode().split(',')
            fixed_values[oid] = value
            if ctx.obj['VERBOSE']:
                click.echo('Fixing {} to value: {}'.format(oid, value))

    try:
        for n in range(0, steps):
            if ctx.obj['VERBOSE']:
                click.echo('Step {}'.format(str(n + 1)))

            # Get currently enterable fields for this subject
            subject_data = get_data(ctx, study, environment, subject)

            subject_data_odm = etree.fromstring(subject_data)
            if subject_data_odm.find('.//' + E_ODM.CLINICAL_DATA.value) is None:
                if ctx.obj['VERBOSE']:
                    click.echo('No data found')
                break

            # Get the metadata version for the subject
            subject_meta_v = subject_data_odm.find('.//' + E_ODM.CLINICAL_DATA.value).get(
                A_ODM.METADATA_VERSION_OID.value)
            if subject_meta_v is None:
                if ctx.obj['VERBOSE']:
                    click.echo('Subject not found')
                break

            # If no metadata supplied, or versions don't match, retrieve metadata from RWS
            if meta_v != subject_meta_v:
                if ctx.obj['VERBOSE']:
                    click.echo('Getting metadata version {}'.format(subject_meta_v))
                ctx.obj['RWS'].send_request(StudyVersionRequest(study, subject_meta_v))
                odm_metadata = ctx.obj['RWS'].last_result.text
                meta_v = subject_meta_v

            # Generate data values to fill in empty fields
            if ctx.obj['VERBOSE']:
                click.echo('Generating data')

            scr = Scramble(odm_metadata)
            odm = scr.fill_empty(fixed_values, subject_data)

            # If new data values, post to RWS
            if etree.fromstring(odm).find('.//' + E_ODM.ITEM_DATA.value) is None:
                if ctx.obj['VERBOSE']:
                    click.echo('No data to send')
                break
            ctx.obj['RWS'].send_request(PostDataRequest(odm))
            if ctx.obj['RAW']:
                click.echo(ctx.obj['RWS'].last_result.text)

    except RWSException as e:
        click.echo(e.rws_error)

    except requests.exceptions.HTTPError as e:
        click.echo(e.strerror)



# -*- coding: utf-8 -*-
import datetime

from rwslib.builders.common import get_utc_date
from rwslib.builders.constants import LocationType, UserType

__author__ = 'glow'

import unittest
from ..common import obj_to_doc
from rwslib.builders.admindata import AdminData, User, FirstName, LastName, Location, DisplayName, MetaDataVersionRef


class TestAdminData(unittest.TestCase):

    def test_create_admin_data(self):
        """Create an AdminData"""
        admin_data = AdminData("Mediflex(Prod)")
        tested = obj_to_doc(admin_data)
        self.assertEqual('Mediflex(Prod)', tested.get('StudyOID'))

    def test_create_admin_data_add_users(self):
        """Create an AdminData and add a User"""
        admin_data = AdminData("Mediflex(Prod)")
        user = User("harold")
        user << FirstName("Harold")
        user << LastName("Kumar")
        admin_data << user
        tested = obj_to_doc(admin_data)
        self.assertEqual('Mediflex(Prod)', tested.get('StudyOID'))
        self.assertEqual('harold', list(tested)[0].get('OID'))
        self.assertEqual('FirstName', list(list(tested)[0])[0].tag)
        self.assertEqual('Harold', list(list(tested)[0])[0].text)

    def test_create_admin_data_add_location(self):
        """Create an AdminData and add a Location"""
        admin_data = AdminData("Mediflex(Prod)")
        location = Location("site1", "Site 1")
        admin_data << location
        tested = obj_to_doc(admin_data)
        self.assertEqual('Mediflex(Prod)', tested.get('StudyOID'))
        self.assertEqual('site1', list(tested)[0].get('OID'))
        self.assertEqual('Site 1', list(tested)[0].get('Name'))

    def test_create_admin_data_and_add_nonsense(self):
        """We can't add what we shouldn't"""
        admin_data = AdminData("Mediflex(Prod)")
        with self.assertRaises(ValueError) as exc:
            admin_data << DisplayName("Happy")
        self.assertEqual("AdminData cannot accept a DisplayName as a child element", str(exc.exception))


class TestLocation(unittest.TestCase):

    def test_create_location(self):
        """Create a Location"""
        obj = Location("Site1", "Site One")
        tested = obj_to_doc(obj)
        self.assertEqual("Location", tested.tag)
        self.assertEqual("Site1", tested.get('OID'))
        self.assertEqual("Site One", tested.get('Name'))

    def test_create_location_of_type(self):
        """Create a Location with typ"""
        obj = Location("Site1", "Site One", location_type=LocationType.Site)
        tested = obj_to_doc(obj)
        self.assertEqual("Location", tested.tag)
        # this is an enum
        self.assertEqual("Site", tested.get('LocationType'))

    def test_create_location_with_metadata_version_refs(self):
        """Create a Location with mutiple MetaDataVersionRef"""
        mdv1 = MetaDataVersionRef(study_oid="Mediflex(Prod)",
                                  metadata_version_oid="1234",
                                  effective_date=datetime.datetime(2017, 1, 1))
        mdv2 = MetaDataVersionRef(study_oid="Mediflex(Prod)",
                                  metadata_version_oid="1235",
                                  effective_date=datetime.datetime(2017, 2, 1))
        obj = Location("Site1",
                       "Site One",
                       location_type=LocationType.Site,
                       metadata_versions=(mdv1, mdv2))
        tested = obj_to_doc(obj)
        self.assertEqual("Location", tested.tag)
        # this is an enum
        self.assertEqual("Site", tested.get('LocationType'))
        self.assertEqual(2, len(tested))

    def test_create_location_with_metadata_version_ref(self):
        """Create a Location with single MetaDataVersionRef"""
        mdv1 = MetaDataVersionRef(study_oid="Mediflex(Prod)",
                                  metadata_version_oid="1234",
                                  effective_date=datetime.datetime(2017, 1, 1))
        mdv2 = MetaDataVersionRef(study_oid="Mediflex(Prod)",
                                  metadata_version_oid="1235",
                                  effective_date=datetime.datetime(2017, 2, 1))
        obj = Location("Site1",
                       "Site One",
                       location_type=LocationType.Site,
                       metadata_versions=mdv1)
        tested = obj_to_doc(obj)
        self.assertEqual("Location", tested.tag)
        # this is an enum
        self.assertEqual("Site", tested.get('LocationType'))
        self.assertEqual(1, len(tested))


class TestUser(unittest.TestCase):

    def test_create_user(self):
        """Create a User"""
        obj = User("user1", user_type=UserType.Investigator)
        tested = obj_to_doc(obj)
        self.assertEqual("User", tested.tag)
        self.assertEqual("user1", tested.get('OID'))
        self.assertEqual("Investigator", tested.get('UserType'))

    def test_create_user_with_display_nae(self):
        """Create a Location"""
        obj = User("user1", user_type=UserType.Investigator)
        obj << DisplayName("Henrik")
        tested = obj_to_doc(obj)
        self.assertEqual("User", tested.tag)
        self.assertEqual("user1", tested.get('OID'))
        self.assertEqual("Investigator", tested.get('UserType'))
        self.assertEqual("DisplayName", list(tested)[0].tag)
        self.assertEqual("Henrik", list(tested)[0].text)


class TestMetaDataVersionRef(unittest.TestCase):

    def test_create_a_version_ref(self):
        """We create a MetaDataVersionRef"""
        _utc_date = get_utc_date()
        obj = MetaDataVersionRef("Mediflex(Prod)", "1024", _utc_date)
        tested = obj_to_doc(obj)
        self.assertEqual("MetaDataVersionRef", tested.tag)
        self.assertEqual("Mediflex(Prod)", tested.get('StudyOID'))
        self.assertEqual("1024", tested.get('MetaDataVersionOID'))
        _effective_date = tested.get('EffectiveDate')
        self.assertIsNotNone(_effective_date)
        _ref_date = get_utc_date().isoformat().split('T')[0]
        self.assertTrue(_effective_date.startswith(_ref_date))

    def test_create_a_version_ref_and_attach_to_location(self):
        """We create a MetaDataVersionRef"""
        this = MetaDataVersionRef("Mediflex(Prod)",
                                  "1024", get_utc_date() - datetime.timedelta(days=7))
        that = MetaDataVersionRef("Mediflex(Prod)", "1025", get_utc_date())
        obj = Location('Site01', 'Site 1')
        obj << this
        obj << that
        tested = obj_to_doc(obj)
        self.assertEqual("Location", tested.tag)
        self.assertTrue(len(list(tested)) == 2)
        _this = list(tested)[0]
        self.assertEqual("Mediflex(Prod)", _this.get('StudyOID'))
        self.assertEqual("1024", _this.get('MetaDataVersionOID'))
        _ref_date = (get_utc_date() - datetime.timedelta(days=7))
        self.assertTrue(_this.get('EffectiveDate').startswith(_ref_date.isoformat().split('T')[0]))
        _that = list(tested)[1]
        self.assertEqual("Mediflex(Prod)", _that.get('StudyOID'))
        self.assertEqual("1025", _that.get('MetaDataVersionOID'))
        _ref_date = get_utc_date()
        self.assertTrue(_that.get('EffectiveDate').startswith(_ref_date.isoformat().split('T')[0]))
